@@ -3,58 +3,10 @@
 #' These functions are imported from other packages and re-exported by
 #' \pkg{plm} to enable smooth use within \pkg{plm}.  Please follow the
 #' links to view the function's original documentation.
-#' 
-#' \itemize{
-#' \item index
-#' \item fixef
-#' \item ranef
-#' \item vcovHC
-#' \item nobs
-#' \item maxLik
-#' }
-#' 
-#' 
-#' 
 #' @name re-export_functions
 #' @keywords internal
 NULL
 
-#' @rdname re-export_functions
-#' @name index
-#' @importFrom zoo index
-#' @export
-NULL
-
-#' @rdname re-export_functions
-#' @name fixef
-#' @importFrom nlme fixef
-#' @export
-NULL
-
-#' @rdname re-export_functions
-#' @name ranef
-#' @importFrom nlme ranef
-#' @export
-NULL
-
-#' @rdname re-export_functions
-#' @name vcovHC
-#' @importFrom sandwich vcovHC
-#' @export
-NULL
-
-#' @rdname re-export_functions
-#' @name lag
-#' @importFrom stats lag
-#' @export
-NULL
-
-#' @rdname re-export_functions
-#' @name nobs
-#' @importFrom stats nobs
-#' @export
-NULL
-
 #' @rdname re-export_functions
 #' @name maxLik
 #' @importFrom maxLik maxLik
@@ -96,7 +48,6 @@ NULL
 #' the package.
 #' 
 #' @name plm-package
-#' @docType package
 #' @keywords package
 #' @examples
 #' 
@@ -124,7 +75,7 @@ NULL
 #'               data = Grunfeld, model = "random", random.method = "walhus",
 #'               effect = "twoways")
 #' 
-NULL
+"_PACKAGE"
 
 
 #' Cigarette Consumption
@@ -173,7 +124,7 @@ NULL
 #' 
 #' Online complements to Baltagi (2013):
 #' 
-#' \url{http://bcs.wiley.com/he-bcs/Books?action=resource&bcsId=4338&itemId=1118672321&resourceId=13452}
+#' \url{https://bcs.wiley.com/he-bcs/Books?action=resource&bcsId=4338&itemId=1118672321&resourceId=13452}
 #' @importFrom Rdpack reprompt
 #' @keywords datasets
 NULL
@@ -266,7 +217,7 @@ NULL
 #' 
 #' Online complements to Baltagi (2013):
 #' 
-#' \url{http://bcs.wiley.com/he-bcs/Books?action=resource&bcsId=4338&itemId=1118672321&resourceId=13452}
+#' \url{https://bcs.wiley.com/he-bcs/Books?action=resource&bcsId=4338&itemId=1118672321&resourceId=13452}
 #' 
 #' See also Journal of Applied Econometrics data archive entry for
 #' Baltagi (2006) at
@@ -341,7 +292,7 @@ NULL
 #' 
 #' Online complements to Baltagi (2013):
 #' 
-#' \url{http://bcs.wiley.com/he-bcs/Books?action=resource&bcsId=4338&itemId=1118672321&resourceId=13452}
+#' \url{https://bcs.wiley.com/he-bcs/Books?action=resource&bcsId=4338&itemId=1118672321&resourceId=13452}
 #' @keywords datasets
 NULL
 
@@ -392,8 +343,9 @@ NULL
 #' 
 #'  website accompanying the paper with various variants of the
 #' Grunfeld data:
-#' \url{https://eeecon.uibk.ac.at/~zeileis/grunfeld/}.
-# \url{http://statmath.wu-wien.ac.at/~zeileis/grunfeld/}.
+#' \url{https://www.zeileis.org/grunfeld/}.
+## \url{https://eeecon.uibk.ac.at/~zeileis/grunfeld/}.
+## \url{http://statmath.wu-wien.ac.at/~zeileis/grunfeld/}.
 #' 
 #' @source Online complements to Baltagi (2001):
 #' 
@@ -403,7 +355,7 @@ NULL
 #' 
 #' Online complements to Baltagi (2013):
 #' 
-#' \url{http://bcs.wiley.com/he-bcs/Books?action=resource&bcsId=4338&itemId=1118672321&resourceId=13452}
+#' \url{https://bcs.wiley.com/he-bcs/Books?action=resource&bcsId=4338&itemId=1118672321&resourceId=13452}
 #' @keywords datasets
 #' @examples
 #' 
@@ -466,7 +418,7 @@ NULL
 #' 
 #' Online complements to Baltagi (2013):
 #' 
-#' \url{http://bcs.wiley.com/he-bcs/Books?action=resource&bcsId=4338&itemId=1118672321&resourceId=13452}
+#' \url{https://bcs.wiley.com/he-bcs/Books?action=resource&bcsId=4338&itemId=1118672321&resourceId=13452}
 #' @keywords datasets
 NULL
 
@@ -630,7 +582,7 @@ NULL
 #' 
 #' Online complements to Baltagi (2013):
 #' 
-#' \url{http://bcs.wiley.com/he-bcs/Books?action=resource&bcsId=4338&itemId=1118672321&resourceId=13452}
+#' \url{https://bcs.wiley.com/he-bcs/Books?action=resource&bcsId=4338&itemId=1118672321&resourceId=13452}
 #' @keywords datasets
 NULL
 
@@ -827,7 +779,7 @@ NULL
 #' 
 #' Online complements to Baltagi (2013):
 #' 
-#' \url{http://bcs.wiley.com/he-bcs/Books?action=resource&bcsId=4338&itemId=1118672321&resourceId=13452}
+#' \url{https://bcs.wiley.com/he-bcs/Books?action=resource&bcsId=4338&itemId=1118672321&resourceId=13452}
 #' @keywords datasets
 #' @examples
 #' 
